package coffeetronix;

import java.awt.Color;
import java.io.File;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.Vector;

/**
 *
 * Central point where the coffeetronix configuration lives.
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 */

public class Config
{
	public static ErrorSystem errinfo= new ErrorSystem();

	public static int field;
	public static int gridSize= 16;

	public static Color gridColor= new Color(235, 235, 235);
	public static Color paperColor= Color.white;

	public static Color linkColor= Color.black;

	public static String fontname= "Times";
	public static int fontsize= 10;

	public static Color shapeColorGray= new Color(230, 230, 230); //gray
	public static Color shapeColorBlue= new Color(220, 235, 247); //blue
	public static Color shapeColorRed= new Color(245, 210, 190); //red	
	public static Color shapeColorGreen= new Color(210, 245, 190); //green	
	public static Color shapeColorYellow= new Color(247, 247, 230); //yellow

	/** Gray color for comment shapes */
	public static Color commentColor= new Color(220, 240, 255);

	/** Color of a java boolean (brown) */
	public static Color boolColor= new Color(120, 120, 0);
	/** Color of a java String (green) */
	public static Color stringColor= new Color(0, 100, 0);
	/** Color of a java byte, int, short, long (pink) */
	public static Color intColor= new Color(100, 0, 100);
	/** Color of a java void (blue) */	
	public static Color voidColor= new Color(0, 0, 100);

	public static int arrowlength= 12;
	public static int arrowwidth= 3;

	private Vector projectlist= new Vector();

	public void rearrangeProjects(String str)
	{
		projectlist.remove(str);
		projectlist.add(str);
	}

	public void addProject(String file)
	{
		// search for duplicate files
		File f0= new File(file);
		File f1;
		for (Iterator iter= projectlist.iterator(); iter.hasNext();)
		{
			String projstr= (String) iter.next();
			f1= new File(projstr);
			if (f1.equals(f0))
				return;
		}
		projectlist.add(file);
	}

	public Iterator getRecentProjects()
	{
		return projectlist.iterator();
	}

	public boolean save(String file)
	{
		try
		{
			FileWriter fileout= new FileWriter(file);
			fileout.write("<!DOCTYPE JECONFIG>\n");
			fileout.write("<ctxconfig>\n");

			int numrecent= 0;
			for (Iterator iter= projectlist.iterator(); iter.hasNext();)
			{
				fileout.write("\t<project>");
				String str= (String) iter.next();
				fileout.write(str);
				fileout.write("</project>\n");

				// maximum of 10 files
				if ((numrecent++) >= 9)
					break;
			}

			fileout.write("</ctxconfig>\n");

			fileout.flush();
			fileout.close();
		}

		// FIXME: Error handler
		catch (Exception e)
		{
			ErrorSystem.error("save-error: exception " + e);
			return false;
		}

		ErrorSystem.info("saved config file: " + file);
		return true;
	}
}
