package coffeetronix;

import java.io.File;
import java.io.FileInputStream;
import java.util.Iterator;
import java.util.LinkedList;

/** Represents a java source file.
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 * 
 */

public class JavaUnit
{

	private File file = null;
	private LinkedList javaclasses = new LinkedList();
	private LinkedList javainterfaces = new LinkedList();
	private LinkedList imports = new LinkedList();
	private String packagestr = "";

	private JavaParser parser = null;


	public String getPackage()
	{
		if (packagestr.length() > 0) return packagestr + ".";
		else return packagestr;
	}

	public void clear()
	{
		packagestr = "";
		imports.clear();
		javainterfaces.clear();
		javaclasses.clear();
	}


	public JavaUnit(File file)
	{
		this.file = file;
	}


	public int parse()
	{
		clear();

		try
		{
			
			if (!parser.isInited)
			{
				parser = new JavaParser(new FileInputStream(file));
				parser.isInited = true;
			}
			else
			{
				parser.ReInit(new FileInputStream(file));
			}

			ErrorSystem.info("parsing java source file:" + file );
			parser.CompilationUnit(this);
			for (Iterator iter = getClasses(); iter.hasNext();)
			{
				JavaClass jclass = (JavaClass) iter.next();
				jclass.fixnestednames("");
			}
			
			return ErrorSystem.OK;
		}
		catch (Exception e)
		{
			ErrorSystem.error("" + e);
			return ErrorSystem.FAILED;
		}
		
	}


	public void setPackage(String str) { packagestr = str; }
	public void addClass(JavaClass jclass) { javaclasses.add(jclass); }
	public void addImport(String importstr) { imports.add(importstr); }
	// FIXME
	public void addInterface(String javainterface) { javainterfaces.add(javainterface); }



	public File getFile()
	{
		return file;
	}

	public Iterator getClasses() { return javaclasses.iterator(); }

	// FIXME:
	public Iterator getInterfaces()
	{
		return null;
	}

}

