package coffeetronix.diagram;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;

import coffeetronix.Config;

/**
 * This is an UML-like graphic representation of a javaclass. 
 * The representation is called shape in the program.
 * It holds its graphics in an java.awt.Image (for effective drawing).
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 */

public class Shape
{
	/** An ugly icon... */
	public static Image boxblueImage;
	/** An ugly icon... */
	public static Image boxblueImageGray;
	/** An ugly icon... */
	public static Image boxpinkImage;
	/** An ugly icon... */
	public static Image boxpinkImageGray;

	/** 
	 * The space between text lines in a class shape can be 
	 * scaled down here */

	private static int fontcompress= 2;
	/** An ugly icon... */
	public static Image privImage;
	/** An ugly icon... */
	public static Image privImageGray;
	/** An ugly icon... */
	public static Image protImage;
	/** An ugly icon... */
	public static Image protImageGray;
	/** An ugly icon... */
	public static Image treeImage;

	/** Shape type definition */
	public static final int TYPE_CLASS= 0;
	/** Shape type definition */
	public static final int TYPE_COMMENT= 1;
	private ShapeInfo collapseddata[];
	private ShapeInfo data[];

	private Dimension dimension = new Dimension();

	private FontMetrics fm;

	/** 
	 * The shape is cached in that image */
	private Image image;

	/** The component that can create Images (Coffeetronix Frame) */
	private Component imagecreator;

	private boolean isdirty= true;
	public boolean isexpanded= true;

	private boolean isselected= false;

	/** Location of the middle point (used as position) */
	private Point midLocation= new Point(getRandom(1000), getRandom(1000));
	private String name="";
	private ShapeInfo normaldata[];
	
	private Rectangle rectangle = new Rectangle();

	/** Color number, see Config.class. */
	private int shapecolor= 0;

	/** Type of the shape, should be subclassed later */
	private int type= 0;

	/** 
	 * Exports the shape data to a textfile for later reading in designlets.
	 */ 
	public void export(FileWriter fw) throws IOException
	{
		fw.write(name + "\n");
		fw.write(isexpanded?"1\n":"0\n");
		fw.write(midLocation.x + "\n");
		fw.write(midLocation.y + "\n");
		fw.write(shapecolor + "\n");
		fw.write(type + "\n");
		fw.write(data.length + "\n");
		for (int n= 0; n < data.length; n++)
		{
			data[n].export(fw);
		}
		fw.write("---\n");
		
	}
	
	
	/** 
	 * Imports data from text file.
	 */
	
	public void imports (BufferedReader br) throws IOException
	{
		String s;
		name = br.readLine();
		s = br.readLine();
		isexpanded = s.startsWith("1");
		int x = Integer.parseInt(br.readLine());
		int y = Integer.parseInt(br.readLine());
		setMidLocation(x,y);
		shapecolor = Integer.parseInt(br.readLine());
		type = Integer.parseInt(br.readLine());
		int len = Integer.parseInt(br.readLine());
		
		normaldata = new ShapeInfo[len];
		for (int n= 0; n < len; n++)
		{
			normaldata[n] = new ShapeInfo("",0,0);
			normaldata[n].imports(br);
		}
		collapseddata = tocollapsed(normaldata);
		// stupid "---"
		br.readLine();
		fixsize();
	}

	/** 
	 * returns angle of a vector related to x axis<br>.
	 * a circle would give values from 0 to 180 and back to 0;
	 */
	private static double getIntersectionAngle180(Dimension dfrom)
	{
		return (180.0 / Math.PI)
			* Math.acos(
				(double) dfrom.width
					/ Math.sqrt(
						(double) (dfrom.width * dfrom.width
							+ dfrom.height * dfrom.height)));
	}

	/** Returns random numbers. */
	private static int getRandom(int range)
	{
		return ((int) (Math.random() * range));
	}

	public boolean contains(Point p)
	{
		return rectangle.contains(p);
	}

	/**  This draws the shape to an image, which is used in the diagram. */
	private Image paintToImage()
	{
		// lazy allocation
		if (image == null)
		{
			image=
				imagecreator.createImage(
					getDimension().width,
					getDimension().height);
		}
		

		Graphics offgraphics= image.getGraphics();
		offgraphics.setFont(imagecreator.getFont());


		// here stupid old geometric stuff
		int w= getDimension().width;
		int h= getDimension().height;

		int x= w / 2;
		int y= h / 2;

		int textx= x - (w - 10) / 2; // 5 pixels border
		int texty= (y - (h - 4) / 2) + fm.getAscent();
		int textheight= fm.getHeight() - fontcompress;

		// class, background color 
		if (type == 0)
		{
			switch (shapecolor)
			{
				case 0 :
					offgraphics.setColor(Config.shapeColorGray);
					break;
				case 1 :
					offgraphics.setColor(Config.shapeColorBlue);
					break;
				case 2 :
					offgraphics.setColor(Config.shapeColorRed);
					break;
				case 3 :
					offgraphics.setColor(Config.shapeColorGreen);
					break;
				case 4 :
					offgraphics.setColor(Config.shapeColorYellow);
					break;
			}
		}

		// comment background
		else
			offgraphics.setColor(Config.paperColor);

		offgraphics.fillRect(x - w / 2, y - h / 2, w - 1, h - 1);

		// black border line
		offgraphics.setColor(Color.black);
		offgraphics.drawRect(x - w / 2, y - h / 2, w - 1, h - 1);

		// comment edge (10 pixels)
		if (type == 1)
			offgraphics.drawLine(w - 10, y - h / 2, w - 1, y - h / 2 + 10);

		for (int line= 0; line < getNumLines(); line++)
		{

			// separator
			if (getType(line) == 10)
			{
				offgraphics.setColor(Color.black);
				offgraphics.drawLine(x - (w / 2), // should be 0 here.
				texty + (line * textheight) - 2, x + w / 2 - 1,
				// should be width - 1
				texty + (line * textheight) - 2);
			}
			else
			{
				Color c;
				// choose some colors
				if (getLine(line).startsWith("void"))
					c= Config.voidColor;
				else if (getLine(line).startsWith("String"))
					c= Config.stringColor;
				else if (getLine(line).startsWith("boolean"))
					c= Config.boolColor;
				else if (
					getLine(line).startsWith("int")
						|| getLine(line).startsWith("long")
						|| getLine(line).startsWith("short")
						|| getLine(line).startsWith("byte"))
					c= Config.intColor;
				else
					c= Color.black;

				// bright text for "disabled" protected elements
				if ((getProtection(line) == 1) || (getProtection(line) == 2))
				{
					Color c1=
						new Color(
							140 + c.getRed() / 4,
							140 + c.getGreen() / 4,
							140 + c.getBlue() / 4);

					offgraphics.setColor(c1);
				}
				else
					offgraphics.setColor(c);

				// text without icon
				if (getType(line) == 0)
				{
					offgraphics.drawString(
						getLine(line),
						textx,
						texty + line * textheight);
				}
				else // icon and right shifted text
					{
					offgraphics.drawString(
						getLine(line),
						20 + textx,
						texty + line * textheight);

					// draw an icon
					switch (getType(line))
					{
						case 0 :
							break;
						case 1 :
							if (getProtection(line) == 0)
							{
								// draw the short blue line to the
								// public method icon
								//offgraphics.setColor(Color.blue);
								/* offgraphics.drawLine(1 + x - (w / 2),
								 should be 1 here.
								texty + (line * textheight) - 5,
									x - (w / 2) + 15,
									texty + (line * textheight) - 5);
								*/
								offgraphics.drawImage(
									boxpinkImage,
									7 + textx,
									texty + (line * textheight) - 10,
									null);

							}
							else // protected, so bright grey "disabled icons"
								offgraphics.drawImage(
									boxpinkImageGray,
									7 + textx,
									texty + (line * textheight) - 10,
									null);

							break;
						case 2 :
							if (getProtection(line) == 0)
							{
								// draw the short blue line to the public
								// attribute icon
								/*
								offgraphics.setColor(Color.blue);
								offgraphics.drawLine(1 + x - (w / 2),
								// should be 1 here.
								texty + (line * textheight) - 5,
									x - (w / 2) + 15,
									texty + (line * textheight) - 5);
								*/
								offgraphics.drawImage(
									boxblueImage,
									7 + textx,
									texty + (line * textheight) - 10,
									null);
							}
							else // privtate, so bright grey "disabled icons"
								offgraphics.drawImage(
									boxblueImageGray,
									7 + textx,
									texty + (line * textheight) - 10,
									null);
							break;

						case 3 :
						case 4 :
							// draw component icon for classes/interfaces
							offgraphics.drawImage(
								treeImage,
								textx,
								texty + (line * textheight) - 10,
								null);
							break;
					}

					switch (getProtection(line))
					{
						case 0 :
							break;
						case 1 :
							// this was the yellow key icon in gray
							offgraphics.drawImage(
								protImageGray,
								textx,
								texty + (line * textheight) - 11,
								null);
							break;
						case 2 :
							// this was the grey lock icon in gray
							offgraphics.drawImage(
								privImageGray,
								textx,
								texty + (line * textheight) - 11,
								null);
							break;
					}
				}
			}
		}

		offgraphics.setColor(Color.red);

		// draw a red rectangle, when the shape is selected.
		if (isselected)
			offgraphics.drawRect(1 + x - w / 2, 1 + y - h / 2, w - 3, h - 3);

		return image;
	}

	/** Calculates the dimension of the shape, 
	 * searches the longes text line.
	 * 
	 */
	public void fixsize()
	{

		if (isexpanded)
		{
			data= normaldata;
		}
		else
		{
			data= collapseddata;
		}

		// find the longest line...
		int maxwidth= 0;
		for (int n= 0; n < data.length; n++)
		{
			// no icon
			if (data[n].getType() == 0)
			{
				if (10 + fm.stringWidth(data[n].getText()) > maxwidth)
					maxwidth= 10 + fm.stringWidth(data[n].getText());
			}
			else // able to shift 20 to right
				{
				if (30 + fm.stringWidth(data[n].getText()) > maxwidth)
					maxwidth= 30 + fm.stringWidth(data[n].getText());
			}
		}

		setDimension(
			new Dimension(
				maxwidth,
				data.length * (fm.getHeight() - fontcompress) + 10));

		setMidLocation(midLocation.x, midLocation.y);

		// image must be drawn again.
		image= null;
	}

	/** Returns the text-line array */
	public ShapeInfo[] getData()
	{
		return data;
	}

	public Dimension getDimension()
	{
		return dimension;
	}

	/** the representation of the shape is drawn to that image */
	public Image getImage()
	{

		if (image == null)
			return paintToImage();
		else
			return image;
	}

	private String getLine(int n)
	{
		try
		{
			return data[n].getText();
		}
		catch (Exception e)
		{
			return "";
		}
	}

	public Point getMidLocation()
	{
		return midLocation;
	}

	/** Returns the internal name of the shape (javaclass name) */
	public String getName()
	{
		return name;
	}

	/** Get number of entries in the class shape. */
	private int getNumLines()
	{
		return data.length;
	}

	/**Protection of entry number n within a shape. */
	private int getProtection(int n)
	{
		try
		{
			return data[n].getProtection();
		}
		catch (Exception e)
		{
			return 0;
		}
	}

	public Rectangle getRectangle()
	{
		return rectangle;
	}

	/** Sets the shape color() */
	public int getShapeColor()
	{
		return this.shapecolor;
	}

	public Point getTopLeft()
	{
		return rectangle.getLocation();
	}

	/** Returns the type of the shape */
	public int getType()
	{
		return type;
	}

	/**  Type of entry number n within a shape.*/
	private int getType(int n)
	{
		try
		{
			return data[n].getType();
		}
		catch (Exception e)
		{
			return 0;
		}
	}

	/** 
	 * Inits a shape with data of a parsed javaclass, font metrics 
	 * and image creater. Init the shape with any textInfo Array, used now for 
	 * comments only. */
	public void init(Component component, String n, int t)
	{
		// this function was called from addCommentShape.
		// type = comment
		// needed in edit popup-menu

		type= t;
		this.name = n;
		this.imagecreator= component;
		this.fm= component.getFontMetrics(component.getFont());
		

		dimension= new Dimension();
		rectangle= new Rectangle();

	}
	
	
	public void setShapeInfo(ShapeInfo info[])
	{

		this.data= info;
		this.collapseddata= tocollapsed(info);
		this.normaldata= data;

		if (isexpanded)
		{
			data= normaldata;
		}
		else
		{
			data= collapseddata;
		}
		fixsize();
	}

	public boolean isDirty()
	{
		return isdirty;
	}

	public boolean isSelected()
	{
		return isselected;
	}

	public void move2D(int dx, int dy)
	{
		isdirty= true;
		rectangle.translate(dx, dy);
		midLocation.translate(dx, dy);
	}

	/** Forces the shape to redraw its shape to the cached image. */
	public void redraw()
	{
		image= null;
	}

	/** Stores the connector information at the place of the partner */
	public void registerLink(LinkInfo linkinfo)
	{
		Point midLocation= linkinfo.shape.getMidLocation();

		int dx= midLocation.x - this.midLocation.x;
		int dy= midLocation.y - this.midLocation.y;

		// Winkel eines Eckpunktes des Rechteckes
		double cornerangle= getIntersectionAngle180(dimension);

		// Winkel der Mittelpunkt-Verbindungsgeraden zur X-Achse
		double toangle= getIntersectionAngle180(new Dimension(dx, dy));
		int newdy, newdx;

		// Strahlensatz: dx/w = dy/h
		newdy= (int) (dy * (dimension.width / 2.0) / Math.abs(dx));
		newdx= (int) (dx * (dimension.height / 2.0) / Math.abs(dy));

		// left
		if (toangle < cornerangle)
		{
			linkinfo.side= 0;
			linkinfo.connector.x= this.midLocation.x + dimension.width / 2;
			linkinfo.connector.y= this.midLocation.y + newdy;
		}
		else if (toangle < (180 - cornerangle))
		{

			if (dy >= 0)
			{
				linkinfo.side= 1; // down
				linkinfo.connector.x= this.midLocation.x + newdx;
				linkinfo.connector.y= this.midLocation.y + dimension.height / 2;
			}
			else
			{
				linkinfo.side= 3; // up
				linkinfo.connector.x= this.midLocation.x + newdx;
				linkinfo.connector.y= this.midLocation.y - dimension.height / 2;
			}
		}
		// toangle > (180 - cornerangle)
		else
		{
			linkinfo.side= 2; // left

			linkinfo.connector.x= this.midLocation.x - dimension.width / 2;
			linkinfo.connector.y= this.midLocation.y + newdy;
		}
	}

	/** Unselect shape (unhighlight its border). */
	public void selectOff()
	{
		isselected= false;
		paintToImage();
	}

	/** Select shape (highlight its border). */
	public void selectOn()
	{
		isselected= true;
		paintToImage();
	}

	/** Toggle shape selection state. */
	public void selectToggle()
	{
		isselected ^= true;
		paintToImage();
	}

	/** Sets the shape size. */
	private void setDimension(Dimension d)
	{
		isdirty= true;
		dimension= d;
		rectangle.setSize(d);
	}

	/** Use when the shapes state or data has changed. */
	public void setDirty(boolean dirty)
	{
		isdirty= dirty;
	}

	public void setMidLocation(int x, int y)
	{
		isdirty= true;
		midLocation.setLocation(x, y);
		rectangle.setLocation(
			x - dimension.width / 2,
			y - dimension.height / 2);
	}
	
	public void setMidLocation(Point p)
	{
		isdirty= true;
		midLocation.setLocation(p.x, p.y);
		rectangle.setLocation(
			p.x - dimension.width / 2,
			p.y - dimension.height / 2);
	}
	

	/** Sets the shape color. */
	public void setShapeColor(int color)
	{
		this.shapecolor= color;
		redraw();
	}

	/** creates a compressed (collapsed, non expanded) version of the shape data*/
	public ShapeInfo[] tocollapsed(ShapeInfo info[])
	{
		LinkedList l= new LinkedList();

		l.add(info[0]); // first line unchanged.
		for (int n= 1; n < info.length; n++)
		{
			ShapeInfo i1= info[n];
			if (i1.getType() == i1.TYPE_SEPARATOR)
			{
				l.add(i1);
				l.add(
					new ShapeInfo(
						"  ...",
						i1.PROT_UNKNOWN,
						i1.TYPE_NORMAL_TEXT));

			}

		}
		return (ShapeInfo[]) l.toArray(new ShapeInfo[l.size()]);
	}

	/** Expands or collapses a shape. */
	public void toggleExpand()
	{

		if (isexpanded)
			isexpanded= false;
		else
			isexpanded= true;

		fixsize();
	}
}
